@@ -1,4 +1,4 @@
-use core::iter::{IntoIterator, Iterator};
+use core::iter::{ExactSizeIterator, IntoIterator, Iterator};
 use core::str;
 
 use thin_vec::ThinVec;
@@ -47,7 +47,16 @@ impl StringsNoIndex {
             let len: u32 = 1;
             self.strs.extend_from_slice(&len.to_ne_bytes());
         } else {
-            self.set_len(self.len() + 1);
+            let len = self.len();
+
+            if len == u32::MAX {
+                panic!(
+                    "StringsNoIndex cannot contain more than u32::MAX {} elements",
+                    u32::MAX
+                );
+            }
+
+            self.set_len(len + 1);
         }
 
         self.strs.extend_from_slice(s.as_bytes());
@@ -106,6 +115,8 @@ impl<'a> Iterator for StringsNoIndexIter<'a> {
             return None;
         }
 
+        self.1 -= 1;
+
         let pos = self.0.iter().position(|byte| *byte == 0).unwrap();
         let slice = &self.0[..pos];
         self.0 = &self.0[(pos + 1)..];
@@ -118,6 +129,8 @@ impl<'a> Iterator for StringsNoIndexIter<'a> {
     }
 }
 
+impl ExactSizeIterator for StringsNoIndexIter<'_> {}
+
 #[cfg(test)]
 mod tests {
     use super::StringsNoIndex;
